///////////////////////////////////////////////////////////////////
// SetColorScheme.cpp
// 720020-1159x
//
// Applied Data Systems
//
// Application loads the UI color scheme information from the registry and
// applied it to the current desktop.  This is only necessary for platforms
// whose shell does not do so automatically.
//
// History
// -------
// 720020-11591: Sep 15, 2003 (ctacke)
//  - Initial implementation
//
// 720020-11592: Nov 16, 2004 (ctacke)
//  - Initial version failed if the "Display Name" of the Scheme didn't
//    match the subkey name of the scheme in the registry, such as for
//    Windows Standard or High Impact White.  This revision fixes that bug.
///////////////////////////////////////////////////////////////////


#include "windows.h"

#define RK_COLORSCHEMES				TEXT("ControlPanel\\Appearance\\Schemes")
#define RK_APPEARANCE				TEXT("ControlPanel\\Appearance")
#define RV_CURRENT					TEXT("Current")

typedef struct 
{
    DWORD 	Version;
    COLORREF argb[C_SYS_COLOR_TYPES];
} SCHEMEDATA, *PSCHEMEDATA;

void SetColorScheme()
{
	DEBUGMSG(TRUE, (_T(" +SetColorScheme\r\n")));
	DWORD	status;
    HKEY    regKey;
	ULONG  	ValLen;
	ULONG 	ValType;
    int 	cColors;
    int 	n, nColors[C_SYS_COLOR_TYPES];
	WCHAR	wSchemeName[100];
    SCHEMEDATA sd;
	COLORREF argb[C_SYS_COLOR_TYPES];

	status = RegOpenKeyEx(HKEY_CURRENT_USER,
                         RK_APPEARANCE,
                         0,
                         0,
                         &regKey);

    if ( status != ERROR_SUCCESS ) 
	{
		DEBUGMSG(TRUE, (_T("   RegOpenKeyEx \"%s\" failed: %i\r\n"), RK_APPEARANCE, status));

		return;
	}
	
	ValLen = sizeof(wSchemeName);
	ValType = REG_SZ;
	status = RegQueryValueEx(regKey,
							RV_CURRENT,
							NULL,
							&ValType,
							(LPBYTE)wSchemeName,
							&ValLen);

	if( status != ERROR_SUCCESS) 
	{
		DEBUGMSG(TRUE, (_T("   RegQueryValueEx \"%s\" failed: %i\r\n"), RV_CURRENT, status));
		goto exit;
	}

   	RegCloseKey(regKey);

	RETAILMSG(TRUE, (_T("  Scheme is currently set to '%s'\r\n"), wSchemeName));

	// the current scheme is the "DisplayName" of the scheme, so we have to iterate to get schemes
	// whose DuispayName don't match the key name (like Windows Standard)

	status = RegOpenKeyEx(HKEY_CURRENT_USER,
                         RK_COLORSCHEMES,
                         0,
                         0,
                         &regKey);

    if ( status != ERROR_SUCCESS ) 
	{
		DEBUGMSG(TRUE, (_T("   RegOpenKeyEx \"%s\" failed: %i\r\n"), RK_COLORSCHEMES, status));
		goto exit;
	}

	int		keyIndex;
	WCHAR	wSubkeyName[100];
	WCHAR	wKeyName[100];
	ULONG	nameLen;
	WCHAR	wDisplayName[100];
	HKEY	subKey;

	keyIndex = 0;

	while(true)
	{
		nameLen = 100;

		RegEnumKeyEx(regKey,
					 keyIndex,
					 wSubkeyName,
					 &nameLen,
					 NULL,
					 NULL,
					 NULL,
					 NULL);

		keyIndex++;

		_tcscpy(wKeyName, RK_COLORSCHEMES);
		_tcscat(wKeyName, _T("\\"));
		_tcscat(wKeyName, wSubkeyName);

		DEBUGMSG(TRUE, (_T("  Checking subkey '%s'\r\n"), wSubkeyName));

		status = RegOpenKeyEx(HKEY_CURRENT_USER,
							 wKeyName,
							 0,
							 0,
							 &subKey);

		if ( status != ERROR_SUCCESS ) 
		{
			DEBUGMSG(TRUE, (_T("   RegOpenKeyEx \"%s\" failed: %i\r\n"), wKeyName, status));
			break;
		}

		ValLen = 100;
		status = RegQueryValueEx(subKey, _T("DisplayName"), NULL, NULL, (PBYTE)wDisplayName, &ValLen); 
		if( status != ERROR_SUCCESS) 
		{
			DEBUGMSG(TRUE, (_T("   RegQueryValueEx failed: %i\r\n"), status));
			break;
		}

		wDisplayName[ValLen] = '\0';

		DEBUGMSG(TRUE, (_T("  DisplayName for '%s' is '%s'\r\n"), wKeyName, wDisplayName));

		if(_tcsicmp(wDisplayName, wSchemeName) == 0)
		{
			DEBUGMSG(TRUE, (_T("  It's a match.  Getting Scheme colors\r\n")));
			// found the match
			ValLen = sizeof(sd);
			status = RegQueryValueEx(subKey, _T("Settings"), NULL, NULL, (PBYTE)&sd, &ValLen); 
			if( status != ERROR_SUCCESS) 
			{
				DEBUGMSG(TRUE, (_T("   RegQueryValueEx failed: %i\r\n"), status));
				break;
			}

			// If scheme in the registry has less colors
			if (ValLen < sizeof(sd)) 
			{
				cColors = C_SYS_COLOR_TYPES - (sizeof(sd)-ValLen)/sizeof(COLORREF);
			}
			else 
			{
				cColors = C_SYS_COLOR_TYPES;
			}

			// Set colors from the registry
			for (n = 0; n < cColors; n++) 
			{
				argb[n] = sd.argb[n];
			}

			// If scheme in the registry has less colors, use
			// current system colors for the missing colors.
			if (cColors < C_SYS_COLOR_TYPES) 
			{
				for (n = cColors; n < C_SYS_COLOR_TYPES; n++) 
				{
					argb[n] = GetSysColor(n | SYS_COLOR_INDEX_FLAG);
				}
			}    

			// Set system colors using current scheme
			for (n=0; n<C_SYS_COLOR_TYPES; n++)
			{
				nColors[n] = n | SYS_COLOR_INDEX_FLAG;
			}

			DEBUGMSG(TRUE, (_T("  Setting System Colors\r\n")));

			SetSysColors(C_SYS_COLOR_TYPES, nColors, argb);

			RegCloseKey(subKey);
			
			break;
		}
		RegCloseKey(subKey);
	}

exit:
   	RegCloseKey(regKey);

	DEBUGMSG(TRUE, (_T(" -SetColorScheme\r\n")));
}

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	RETAILMSG(TRUE, (_T("Setting Color Scheme...\r\n")));
	SetColorScheme();
	RETAILMSG(TRUE, (_T("Done setting color scheme.\r\n")));
	return 0;
}
